@@ -25,13 +25,13 @@ def display(params, title=None):
     param_values[rate_mask] = 10 ** local_params
     traj = solver.run(param_values=param_values)
     ysim_array = traj.dataframe[obs_names].values
-    # ysim_array = normalize(ysim_array)
+    ysim_norm = normalize(ysim_array)
 
     plt.figure(figsize=(6, 4))
     if title is not None:
         plt.title(title)
-    plt.plot(t, ysim_array[:, 0], '^r', linestyle='--', label='A')
-    plt.plot(t, ysim_array[:, 1], 'ok', linestyle='--', label='C')
+    plt.plot(t, ysim_norm[:, 0], '^r', linestyle='--', label='A')
+    plt.plot(t, ysim_norm[:, 1], 'ok', linestyle='--', label='C')
     plt.plot(t, norm_noisy_data_A, 'r-', label='Noisy A')
     plt.plot(t, norm_noisy_data_C, 'k-', label='Noisy C')
     plt.legend(loc=0)
@@ -109,8 +109,7 @@ if '__main__' == __name__:
 
     # Here we initial the class
     # We must proivde the cost function and a starting value
-    optimizer = PSO(cost_function=obj_function, start=start_position,
-                    verbose=True, shrink_steps=False)
+    optimizer = PSO(start=start_position, verbose=True, shrink_steps=False)
 
     # We also must set bounds of the parameter space, and the speed PSO will
     # travel (max speed in either direction)
@@ -118,8 +117,8 @@ if '__main__' == __name__:
     optimizer.set_speed(speed_min=-.05, speed_max=.05)
 
     # Now we run the pso algorithm
-    optimizer.run(num_particles=50, num_iterations=500, num_processes=12,
-                  max_iter_no_improv=25)
+    optimizer.run(num_particles=50, num_iterations=500, num_processors=12,
+                  cost_function=obj_function, max_iter_no_improv=25)
 
     best_params = optimizer.best.pos
     display(best_params, "After optimization")
