@@ -1,24 +1,38 @@
 # -*- coding: utf-8 -*-
-from concurrent.futures import ProcessPoolExecutor
-from copy import deepcopy
 import os
+from concurrent.futures import ProcessPoolExecutor, Executor, Future
+from copy import deepcopy
 
 import numpy as np
 
-from pysb.simulator.scipyode import SerialExecutor
 from simplepso.logging import setup_logger
 
+# set OMP_NUM_THREADS to 1 to prevent multi-processing to expand no each core
 os.environ['OMP_NUM_THREADS'] = "1"
 
 
 class Particle(object):
-    def __init__(self, pos):
-        self.pos = pos
-        self.fitness = None
-        self.best = None
-        self.smin = None
-        self.smax = None
+    """
+    Particle to be used in the Swarm
+    """
 
+    def __init__(self, pos, fitness=None, smin=None, smax=None):
+        self.pos = pos
+        self.fitness = fitness
+        self.smin = smin
+        self.smax = smax
+        self.__best = None
+
+    @property
+    def best(self):
+        return self.__best
+
+    @best.setter
+    def best(self, new_best):
+        self.__best = Particle(
+            deepcopy(new_best.pos), deepcopy(new_best.fitness),
+            deepcopy(new_best.smin), deepcopy(new_best.smax)
+        )
 
 # Used for printing below
 header = '{:<10}' + "\t".join(['{:>12}'] * 5)
@@ -27,53 +41,42 @@ stats_output = '{:<10}' + '\t'.join(['{:>12.3f}'] * 5)
 
 
 class PSO(object):
-    """ Simple interface to run particle swarm optimization
-
-    This class provides a simple interface to run particle swarm optimization.
-    It builds off the deap package, but provides a simple interface.
-
-    Examples:
-
-        optimizer = simplepso.pso.PSO(cost_function,start_position)
-        optimizer.set_bounds()
-        optimizer.set_speed()
-        optimizer.run(number_of_particles, number_of_iterations, num_processes)
-
-
-    Notes:
-        Must set 1.) cost_function, 2.) starting position, 3.) bounds
-        To run need to supply number of particles and number of iterations.
-        20 particles is a good starting place.
+    """
+    Simple interface to run particle swarm optimization
 
+    It can optimize parameters for a function using two run methods.
+        run() : cost function gets a parameter vector and returns a scalar. Can
+            be used with any callable function.
+        run_ssa() : cost function gets multiple trajectories of a PySB model
+            and returns a scalar. These trajectories are provided as
+            a pandas.DataFrame. PySB dependent function.
     """
 
-    def __init__(self, cost_function=None, start=None,
-                 save_sampled=False, verbose=False, shrink_steps=True,
-                 simulator=None):
+    def __init__(self, start=None, save_sampled=False, verbose=False,
+                 shrink_steps=True):
         """
 
         Parameters
         ----------
-        cost_function : function
-            Callable function that takes a parameter and returns a tuple
-        start : list_list
+        start : list
             Starting position
         save_sampled : bool
-            Save each position samples
+            Save each particles position and fitness over all iterations.
         verbose : bool
-
+        shrink_steps : bool
+            Shrink max distance traveled by each particle as the number of
+            iterations increases
         """
-        self.cost_function = cost_function
         self.save_sampled = save_sampled
         if start is not None:
             self.set_start_position(start)
         else:
             self.start = None
+            """Starting position"""
             self.size = None
-        if simulator is not None:
-            self.simulator = simulator
         self.verbose = verbose
         self.best = None
+        """Best Particle of population"""
         self.max_speed = None
         self.min_speed = None
         self.lb = None
@@ -81,11 +84,19 @@ class PSO(object):
         self.bounds_set = False
         self.range = 2
         self.all_history = None
+        """History of all particles positions over all iterations. Only saved
+        if save_sampled=True"""
         self.all_fitness = None
+        """History of all particles finesses over all iterations. Only saved
+        if save_sampled=True"""
         self.population = []
+        """Population of particles"""
         self.population_fitness = []
+        """Fitness values of population of particles"""
         self.values = []
+        """Fitness values of the best particle for each iteration"""
         self.history = []
+        """Best particle for each iteration"""
         self.update_w = shrink_steps
         self._is_setup = False
         self.log = setup_logger(verbose)
@@ -109,16 +120,6 @@ class PSO(object):
         part.smax = self.max_speed
         return part
 
-    def set_cost_function(self, cost_function):
-        """ Cost function must return a scalar
-
-        Parameters
-        ----------
-        cost_function : callable function
-
-        """
-        self.cost_function = cost_function
-
     def _setup_pso(self):
         """ Sets up everything for PSO. Only does once """
         if self.max_speed is None or self.min_speed is None:
@@ -131,21 +132,24 @@ class PSO(object):
             raise Exception("Must provide a starting position\n"
                             "Use PSO.set_start_position() \n")
 
-        if self.cost_function is None:
-            raise Exception("No cost function. "
-                            "Set with PSO.set_cost_function().")
-
         self._is_setup = True
 
     def _update_connected(self):
         """ Update the population of particles"""
         for part in self.population:
             if part.best is None or part.best.fitness > part.fitness:
-                part.best = deepcopy(part)
-                part.best.fitness = deepcopy(part.fitness)
+                part.best = part
+                # part.best = deepcopy(part)
+                # part.best.fitness = deepcopy(part.fitness)
             if self.best is None or self.best.fitness > part.fitness:
-                self.best = deepcopy(part)
-                self.best.fitness = deepcopy(part.fitness)
+                self.best = Particle(
+                    deepcopy(part.pos),
+                    deepcopy(part.fitness),
+                    deepcopy(part.smin),
+                    deepcopy(part.smax)
+                )
+                # self.best = deepcopy(part)
+                # self.best.fitness = deepcopy(part.fitness)
 
     def _update_particle_position(self, part):
         """ Updates an individual particles position """
@@ -172,16 +176,20 @@ class PSO(object):
         -------
         np.array, np.array
         """
-        positions = np.zeros(np.shape(self.population))
-        fitnesses = np.zeros(len(self.population))
+        positions = []
+        finesses = []
         for n, part in enumerate(self.population):
-            fitnesses[n] = part.best.fitness
-            positions[n] = part.best.pos
-        idx = np.argsort(fitnesses)
-        return fitnesses[idx], positions[idx]
+            finesses.append(part.best.fitness)
+            positions.append(part.best.pos)
+
+        positions = np.array(positions)
+        finesses = np.array(finesses)
+        idx = np.argsort(finesses)
+        return finesses[idx], positions[idx]
 
     def set_start_position(self, position):
-        """ Set the starting position for the population of particles.
+        """
+        Set the starting position for the population of particles.
 
         Parameters
         ----------
@@ -194,8 +202,11 @@ class PSO(object):
     def set_speed(self, speed_min=-10000, speed_max=10000):
         """ Sets the max and min speed of the particles.
 
-        This is usually a fraction of the range of values that the particles
-        can travel.
+        This is usually a fraction of the bounds set with `set_bounds`. So if
+        one sets the bound to be +/- 1 order of magnitude, you can set the
+        speed to be -.1 and .1, allow the particles to update in 1/10 the
+        parameter space. This keeps particles near their local position rather
+        than jumping across parameter space quickly.
 
         Parameters
         ----------
@@ -216,8 +227,13 @@ class PSO(object):
         Parameters
         ----------
         parameter_range : float
-        lower : array of lower bounds for parameters
-        upper : array of upper bounds for parameters
+            If provided parameter_range, the range will be set by the starting
+            position +/- this value. To set each parameter manually, use
+            `lower` and `upper` args
+        lower : array
+            Lower bounds for parameters
+        upper : array
+            Upper bounds for parameters
 
         """
         if self.start is None:
@@ -245,16 +261,21 @@ class PSO(object):
         self.ub = upper
         self.bounds_set = True
 
-    def run(self, num_particles, num_iterations, num_processes=1,
-            save_samples=False, stop_threshold=1e-5, max_iter_no_improv=None):
-        """ Run optimization
+    def run(self, num_particles, num_iterations, cost_function=None,
+            num_processors=1, save_samples=False, stop_threshold=1e-5,
+            max_iter_no_improv=None):
+        """
+        Run optimization
 
         Parameters
         ----------
         num_particles : int
             Number of particles in population, ~20 is a good starting place
         num_iterations : int
-        num_processes : int
+            Number of iterations to perform.
+        cost_function : callable function
+            Takes a parameter set and returns a scalar (particles fitness)
+        num_processors : int
             Number of processors to run on. If using scipy, note that you may
             need to set OMP_NUM_THREADS=1 to prevent each process from using
             more than one CPU.
@@ -262,15 +283,20 @@ class PSO(object):
             Save ALL positions of particles over time, can require large memory
             if num_particles, num_iterations, and len(parameters) is large.
         stop_threshold : float
-            Threshold of standard deviation of all particles cost function.
+             Standard deviation of the particles’ cost function at which the
+             optimization is stopped
         max_iter_no_improv: int
-            Maximum steps allowed without improvement
+            Maximum steps allowed without improvement before the optimization
+            stops.
         """
         if self._is_setup:
             pass
         else:
             self._setup_pso()
 
+        if not callable(cost_function):
+            raise TypeError("Provide a callable cost function")
+
         history = np.zeros((num_iterations, len(self.start)))
         if self.save_sampled or save_samples:
             self.all_history = np.zeros(
@@ -283,14 +309,14 @@ class PSO(object):
             max_iter_no_improv = np.inf
         iter_without_improvement = 0
         best_fitness = np.inf
-        with SerialExecutor() if num_processes == 1 else \
-                ProcessPoolExecutor(max_workers=num_processes) as executor:
+        with SerialExecutor() if num_processors == 1 else \
+                ProcessPoolExecutor(max_workers=num_processors) as executor:
             for g in range(num_iterations):
                 if self.update_w:
                     self.w = (num_iterations - g + 1.) / num_iterations
                 self.population_fitness = np.array(
                     list(executor.map(
-                        self.cost_function,
+                        cost_function,
                         [p.pos for p in self.population],
                     )
                     )
@@ -325,7 +351,7 @@ class PSO(object):
         self.values = np.array(values[:g])
         self.history = np.array(history[:g, :])
 
-    def _calc_fitness_from_array(self, traj, num_sim):
+    def _calc_fitness_from_array(self, traj, num_sim, cost_function):
         index_names = traj.index.names
         traj.reset_index(inplace=True)
         for i, part in enumerate(self.population):
@@ -333,9 +359,8 @@ class PSO(object):
             end = (i + 1) * num_sim
             tmp_results = traj.loc[
                 traj.simulation.isin(list(range(start, end)))]
-            # tmp_results = tmp_results.T.unstack('simulation')
             tmp_results.set_index(index_names, inplace=True)
-            error = self.cost_function(tmp_results)
+            error = cost_function(tmp_results)
             part.fitness = error
             self.population_fitness[i] = error
 
@@ -350,25 +375,35 @@ class PSO(object):
         return rate_vals
 
     def run_ssa(self, model, num_particles, num_iterations, num_sim,
-                save_samples=False, stop_threshold=0):
-        """ Run PSO for a stochastic simulator
+                cost_function, simulator, save_samples=False,
+                stop_threshold=0):
+        """
+        Run PSO for a stochastic simulator
 
         Parameters
         ----------
         model : pysb.Model
         num_particles : int
+            Number of particles in the swarm.
         num_iterations : int
+            Number of iterations to perform
         num_sim : int
+            Number of SSA simulations to run for each particle.
+        cost_function : function
+            Takes a pandas dataframe of PySB trajectories created by running
+            multiple SSA simulations. Function must return a scalar.
+        simulator : pysb.Simulator
+            PySB simulator (CudaSSASimulator or OpenclSSASimulator)
         save_samples : bool
         stop_threshold : float
-
         """
         if self._is_setup:
             pass
         else:
             self._setup_pso()
-        if self.simulator is None:
-            raise Exception("Must provide an SSA simulator to use this method")
+        if simulator is None:
+            raise ValueError(
+                "Must provide an SSA simulator to use this method")
 
         history = []
         values = []
@@ -392,17 +427,16 @@ class PSO(object):
             all_param_vals[:, rate_mask] = 10 ** rate_values
             # reset initials and param_values so simulator won't try to
             # duplicate any
-            self.simulator.initials = None
-            self.simulator.param_values = None
-            traj = self.simulator.run(
-                param_values=all_param_vals).dataframe  # .T
-            self._calc_fitness_from_array(traj, num_sim)
+            simulator.initials = None
+            simulator.param_values = None
+            traj = simulator.run(param_values=all_param_vals).dataframe
+            self._calc_fitness_from_array(traj, num_sim, cost_function)
             self._update_connected()
             for part in self.population:
                 self._update_particle_position(part)
 
             values.append(self.best.fitness)
-            history.append(self.best.pos)
+            history.append(deepcopy(self.best.pos))
 
             if self.save_sampled or save_samples:
                 curr_fit, curr_pop = self.return_ranked_populations()
@@ -415,8 +449,8 @@ class PSO(object):
                 self.log.info("Stopping criteria reached.")
                 break
 
-        self.values = values
-        self.history = history
+        self.values = np.array(values)
+        self.history = np.array(history)
 
     def print_stats(self, iteration, fitness):
         if iteration == 1:
@@ -425,3 +459,20 @@ class PSO(object):
             stats_output.format(iteration, self.best.fitness, fitness.mean(),
                                 fitness.min(), fitness.max(), fitness.std())
         )
+
+
+class SerialExecutor(Executor):
+    """ Execute tasks in serial (immediately on submission)
+    Code originally from PySB.
+    """
+
+    def submit(self, fn, *args, **kwargs):
+        f = Future()
+        try:
+            result = fn(*args, **kwargs)
+        except BaseException as e:
+            f.set_exception(e)
+        else:
+            f.set_result(result)
+
+        return f
